#include <iostream>
#include <vector>
using namespace std;

template <typename T> void negate_all(vector<T>& v) {
	for_each(v.begin(), v.end(), [](T& n) { n = -n; });
}

template <typename T> void print_all(const vector<T>& v) {
	for_each(v.begin(), v.end(), [](const T& n) { cout << n << " "; });
	cout << endl;
}

int main() {
	vector<int> v {34, -45, 73};
	print_all(v);
	negate_all(v);
	cout << "After negate_all():" << endl;
	print_all(v);
	cout << endl;

	vector<double> vd {3.4, -4.5, 7.3};
	print_all(vd);
	negate_all(vd);
	cout << "After negate_all():" << endl;
	print_all(vd);
	cout << endl;

//	C++-14
//	Lambda function parameters can now be auto to let the compiler deduce the
//	type. This generates a lambda type with a templated operator() so that the
//	same lambda object can be invoked with any suitable type and a type-safe
//	function with the right parameter type will be automatically generated.

	auto divide_by_5 = [](auto x) { cout << x / 5 << endl; };
	divide_by_5(1);
	divide_by_5(1.);
	cout << endl;

	auto print_element = [](auto n) { cout << n << " "; };
	for_each(v.begin(), v.end(), print_element);
	cout << endl;
	for_each(vd.begin(), vd.end(), print_element);
	cout << endl;
}

